<?php

namespace Drupal\terms_export_import\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\terms_export_import\Service\TermsExportImportService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Import form for terms.
 */
class TermExportForm extends FormBase {

  /**
   * The terms export import service.
   *
   * @var \Drupal\terms_export_import\Service\TermsExportImportService
   */
  protected TermsExportImportService $exportImportService;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Constructs a new TermExportForm.
   *
   * @param \Drupal\terms_export_import\Service\TermsExportImportService $exportImportService
   *   The terms export import service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(TermsExportImportService $exportImportService, EntityTypeManagerInterface $entityTypeManager) {
    $this->exportImportService = $exportImportService;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('terms_export_import.service'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'terms_export_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['vocabulary'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Vocabulary'),
      '#target_type' => 'taxonomy_vocabulary',
      '#required' => TRUE,
      '#description' => $this->t('Select the vocabulary to export.'),
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Export CSV'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $vocabulary_id = $form_state->getValue('vocabulary');
    $vocab = $this->entityTypeManager->getStorage('taxonomy_vocabulary')->load($vocabulary_id);
    if (!$vocab) {
      $this->messenger()->addError($this->t('Invalid vocabulary selected.'));
      return;
    }

    $terms_data = $this->exportImportService->exportTerms($vocab->id());
    $csv = $this->exportImportService->convertToCsv($terms_data);

    // Return as CSV file download.
    $response = new Response($csv);
    $response->headers->set('Content-Type', 'text/csv; charset=UTF-8');
    $filename = $vocab->id() . '-terms-export.csv';
    $response->headers->set('Content-Disposition', 'attachment; filename="' . $filename . '"');
    $form_state->setResponse($response);
  }

}
